unit principale;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  System.Math.Vectors, FMX.Controls3D, FMX.MaterialSources, FMX.Objects3D,
  GBECubemap, FMX.Viewport3D, FMX.Ani, GBEHeightmap, FMX.Layouts,
  FMX.Types3D, FMX.Controls.Presentation, FMX.StdCtrls;

type
  TfPrincipale = class(TForm)
    Viewport3D: TViewport3D;
    GBECubemap: TGBECubemap;
    TextureMaterialSource: TTextureMaterialSource;
    dmyJoueurOrientation: TDummy;
    dmyJoueur: TDummy;
    Camera1: TCamera;
    Layout1: TLayout;
    procedure FormCreate(Sender: TObject);
    procedure Layout1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure Layout1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
  private
    { Dclarations prives }
    FPosDepartCurseur: TPointF;    // Position du pointeur de souris au dbut du mouvement de la souris
    procedure SetAngleDeVue(const Value: TPointF); // Modification de l'angle de vue
  public
    { Dclarations publiques }
    property posDepartCurseur: TPointF read FPosDepartCurseur write FPosDepartCurseur; // Proprit de la position du pointeur de souris au dbut du mouvement de la souris
    property angleDeVue : TPointF write SetAngleDeVue; // Proprit de l'angle de vue
  end;

var
  fPrincipale: TfPrincipale;

implementation

{$R *.fmx}

procedure TfPrincipale.FormCreate(Sender: TObject);
begin
  GBECubemap.generateCubemap;
end;

procedure TfPrincipale.SetAngleDeVue(const Value: TPointF);
var
  ptA,ptD,S : TPointF; // ptA point d'arriv, ptD point de dpart, S la sensibilit
begin
  S.X := 180 / viewport3D.Width; // Rglage de la sensibilit pour l'orientation droite/gauche
  S.Y := 180 / viewport3D.Height;// Rglage de la sensibilit pour l'orientation haut/bas
  ptA := Value * S;            // Point d'arrive adapt  la sensibilit
  ptD := posDepartCurseur * S; // Point de dpart adapt  la sensibilit
  // Vue droite/gauche
  with dmyJoueurOrientation.RotationAngle do y := y + (ptA.X - ptD.X); // orientation droite/gauche (axe y) en fonction du dplacement de la souris en X
  // Vue Haut/Bas
  with dmyJoueur.RotationAngle do x:= x + (ptD.Y - ptA.Y); // de mme pour l'orientation haut/bas en adaptant (rotation sur l'axe x, e fonction du d'dplacement de la souris en Y
  posDepartCurseur := Value;   // la position du curseur lorsque l'utilisateur a cliqu (l'origine de la direction), est mis  jour avec la nouvelle position du curseur : au prochain appel de OnMouseMove, la position de dpart doit tre la position d'arrive du coup prcdent
end;

procedure TfPrincipale.Layout1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Single);
begin
  if ssLeft in shift then posDepartCurseur := PointF(X,Y);
end;

procedure TfPrincipale.Layout1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Single);
begin
  if ssLeft in shift then angleDeVue := PointF(X,Y);
end;

end.
